package ascio.lib;

import com.ascio.www._2007._01.Domain;
import com.ascio.www._2007._01.IAscioServices;
import com.ascio.www._2007._01.Message;
import com.ascio.www._2007._01.Order;
import com.ascio.www._2007._01.OrderStatusType;
import com.ascio.www._2007._01.RegistrantVerificationDetails;
import com.ascio.www._2007._01.Response;
import com.ascio.www._2007._01.holders.DomainHolder;
import com.ascio.www._2007._01.holders.OrderHolder;
import com.ascio.www._2007._01.holders.ResponseHolder;
import java.io.PrintStream;
import java.util.Properties;

public class V2 extends ApiVersion
{
  private String sessionId;
  private IAscioServices client = null;
  private java.io.File examplesConfigFile = new java.io.File("examples.properties");
  private Properties properties = new Properties();
  
  public V2(ServerConfig serverConfig) { super(serverConfig); }
  
  public IAscioServices getClient() throws Exception {
    if (client != null) {
      return client;
    }
    com.ascio.www._2012._01._01.AscioService.AscioServicesLocator locator = new com.ascio.www._2012._01._01.AscioService.AscioServicesLocator();
    locator.setBasicHttpBinding_IAscioServicesEndpointAddress(getEndpoint());
    client = locator.getBasicHttpBinding_IAscioServices();
    
    com.ascio.www._2007._01.Session session = new com.ascio.www._2007._01.Session();
    session.setAccount(serverConfig.getAccount());
    session.setPassword(serverConfig.getPassword());
    
    ResponseHolder responseHolder = new ResponseHolder();
    javax.xml.rpc.holders.StringHolder sessionIdHolder = new javax.xml.rpc.holders.StringHolder();
    client.logIn(session, responseHolder, sessionIdHolder);
    System.out.println("Endpoint\t\t: " + getEndpoint());
    System.out.println("Login Response\t\t: " + responseHolder.value.getMessage());
    System.out.println("Login Response code\t: " + responseHolder.value.getResultCode());
    if (responseHolder.value.getResultCode().shortValue() == 401) {
      throw new Exception("Ascio LogIn, Code: " + responseHolder.value.getResultCode() + ", " + responseHolder.value.getMessage());
    }
    System.out.println("SessionId\t\t: " + sessionIdHolder.value);
    
    sessionId = sessionIdHolder.value;
    return client;
  }
  

  public String getSessionId() { return sessionId; }
  
  public Domain repeatSync(String orderId, Integer iterations) throws java.rmi.RemoteException, InterruptedException {
    AscioConfig config = new AscioConfig();
    ResponseHolder getOrderResult = new ResponseHolder();
    OrderHolder order = new OrderHolder();
    client.getOrder(getSessionId(), orderId, getOrderResult, order);
    OrderStatusType status = order.value.getStatus();
    System.out.println("sync " + iterations + "\t\t\t: Order " + status);
    if (status == OrderStatusType.Completed) {
      DomainHolder domain = new DomainHolder();
      ResponseHolder getDomainResult = new ResponseHolder();
      String domainHandle = order.value.getDomain().getDomainHandle();
      System.out.println("Domain handle\t\t: " + domainHandle);
      client.getDomain(getSessionId(), domainHandle, getDomainResult, domain);
      System.out.println("Domain name\t\t: " + domain.value.getDomainName());
      java.text.SimpleDateFormat dt = new java.text.SimpleDateFormat("dd-mm-yyyy");
      java.util.Date expDate = domain.value.getExpDate().getTime();
      System.out.println("Expire date\t\t: " + dt.format(expDate));
      System.out.println("Domain status\t\t: " + domain.value.getStatus());
      writeProperties(order.value, domain.value);
      return domain.value;
    }
    
    if ((status != OrderStatusType.Failed) && 
      (status != OrderStatusType.Invalid))
    {
      if (iterations.intValue() == 0) {
        System.out.println("interrupted sync\t:" + status);
        return null;
      }
      if (status == OrderStatusType.Pending_End_User_Action) {
        System.out.println("Please confirm the e-mail that was sent to: " + config.getEmail());
      }
      Thread.sleep(3000L);
      return repeatSync(orderId, Integer.valueOf(iterations.intValue() - 1));
    }
    return null;
  }
  
  public void initTests() throws Exception { client = getClient();
    RegistrantVerificationDetails details = new RegistrantVerificationDetails();
    Message message = new Message();
    message.setBody("This is the body");
    message.setToAddress("administrator@ascio-test-domain.com");
    message.setSubject("My test subject");
    message.setType(com.ascio.www._2007._01.MessageType.Message_to_Registrant);
    message.setFromAddress("me@ascio-partner.com");
    details.setVerifiedBy("5.6.7.8");
    details.setVerificationDate(java.util.Calendar.getInstance());
    details.setMessages(new Message[] { message });
    Response response = client.uploadRegistrantVerificationMessage(getSessionId(), "administrator@ascio-test-domain.com", details);
    System.out.println("UploadRegistrantVerificationMessage Message\t: " + response.getMessage());
    System.out.println("UploadRegistrantVerificationMessage Code\t: " + response.getResultCode());
  }
  
  public Properties getProperties() {
    if (properties.getProperty("DomainHandle") != null) {
      return properties;
    }
    try {
      java.io.FileInputStream is = new java.io.FileInputStream(examplesConfigFile);
      properties.loadFromXML(is);
    } catch (Exception e) {
      e.printStackTrace();
    }
    return properties;
  }
  
  public void writeProperties(Order order, Domain domain) { if (order.getStatus() != OrderStatusType.Completed) {
      return;
    }
    try {
      properties = new Properties();
      properties.setProperty("OrderId", order.getOrderId());
      properties.setProperty("DomainHandle", domain.getDomainHandle());
      properties.setProperty("DomainName", domain.getDomainName());
      properties.setProperty("RegistrantHandle", domain.getRegistrant().getHandle());
      properties.setProperty("ContactHandle", domain.getAdminContact().getHandle());
      properties.setProperty("NameserverHandle", domain.getNameServers().getNameServer1().getHandle());
      java.io.FileOutputStream os = new java.io.FileOutputStream(examplesConfigFile);
      properties.storeToXML(os, "Generated by V2.writeProperties at successful registration.");
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}